function NonOvlpHarm = FindNonOverlapHarmonics(F0s, maxHarm, harmOvlpTh)

% Input
%   - F0s           : pitches (Hz, a row vector)
%   - maxHarm       : the number of harmonics to consider
%   - harmOvlpTh    : threshold to determine if two harmonics are overlapped (Hz)
% Output
%   - NonOvlpHarm   : non-overlapped harmonics (each column corresponds to a pitch)

nF0 = length(F0s);
NonOvlpHarm = repmat((1:maxHarm)', 1, nF0);

if nF0 == 1
    return;
end

for i = 1:nF0
    pitch = F0s(i);
    idx = (F0s > pitch);            % find the other pitches that are larger than this one
    nInterfer = length(F0s(idx));   % number of intereference F0s
    
    if nInterfer == 0
        continue;
    end
    
    % a row vector storing all the possible interference frequencies
    InterferFreq = reshape(F0s(idx) * (1:maxHarm), 1, nInterfer*maxHarm);    
    
    % a distance matrix between the harmonic frequencies of the target
    % pitch and those of the interference pitches
    disMat = abs(repmat(pitch*(1:maxHarm)', 1, nInterfer*maxHarm) - repmat(InterferFreq, maxHarm, 1));
    
    [minDis, ~] = min(disMat, [], 2);
    
    idx = minDis < harmOvlpTh;
    
    NonOvlpHarm(idx, i) = 0;
end
    

