% SVM classification
% code for paper: Zhiyao Duan, Bryan Pardo, and Laurent Daudet, "A novel cepstral
%   representation for timbre modeling of sound sources in polyphonic
%   mixtures," in Proc. IEEE International Conference on Acoustics, Speech and
%   Signal Processing (ICASSP), 2014.

% to generate results of other features, replace "MUDC" to other feature
% names "OC", "MFCC", "DC", "RDC", "UDC", "HS". There are 7 places for the
% replacement in the code.

clear;
clc;
rng('shuffle');

for poly = 2:6
Acc_MUDC = zeros(10, 8);

for iter = 1:10

    %% create training data
    [trainLabel, trainFeature] = createTrainData('MUDC', 50, 5);

    %% scale the training data to [-1 1]
    minTrainFeature = min(trainFeature, [], 1);
    ranTrainFeature = max(trainFeature, [], 1) - min(trainFeature, [], 1);
    scaledTrainFeature = 2*(trainFeature - repmat(minTrainFeature, size(trainFeature,1), 1))...
        ./ repmat(ranTrainFeature, size(trainFeature,1), 1) - 1;

    %% create test data
    [testLabel, testFeature, testPitch] = createTestData_RWC('MUDC', 1000, poly, 50);

    %% scale the test data using the same linear transformation
    scaledTestFeature = 2*(testFeature - repmat(minTrainFeature, size(testFeature,1), 1))...
        ./ repmat(ranTrainFeature, size(testFeature,1), 1) - 1;

    %% save train and test data
    save(sprintf('Scaled_TrainIso_TestMix_MUDC_poly%d_iter%d', poly, iter), 'scaledTrainFeature', 'trainLabel', 'scaledTestFeature', 'testLabel');

    for p = 5:5:40
        %% SVM train
        % model = svmtrain(trainLabel, scaledTrainFeature(:, 1:40), '-b 1 -c 100 -v 5');
        model = svmtrain(trainLabel, scaledTrainFeature(:, 1:p), '-b 1 -c 100');

        %% SVM prediction
        [predicted_label, accuracy, prob_estimates] = svmpredict(testLabel, scaledTestFeature(:, 1:p), model, '-b 1');

        Acc_MUDC(iter, p/5) = accuracy(1);
    end    
    
end
save(sprintf('Acc_MUDC_poly%d.mat',poly), 'Acc_MUDC');
end