% ShowPaperResults

%% Acc (mean and std) vs. dimensionality of different methods
% train on poly2lated signal
load('results_trainiso/Acc_OC_poly4.mat');
load('results_trainiso/Acc_MFCC_poly4.mat');
load('results_trainiso/Acc_DC_poly4.mat');
load('results_trainiso/Acc_RDC_poly4.mat');
load('results_trainiso/Acc_UDC_poly4.mat');
load('results_trainiso/Acc_MUDC_poly4.mat');
load('results_trainiso/Acc_HS_poly4.mat');

x = 5:5:40;
figure; h = axes('FontSize', 16); hold on; grid on;
plot(x, mean(Acc_OC,1), 'b--^', 'LineWidth', 2);
plot(x, mean(Acc_MFCC,1), 'b-^', 'LineWidth', 2);
plot(x, mean(Acc_DC,1), 'g--o', 'LineWidth', 2);
plot(x, mean(Acc_RDC,1), 'g-o', 'LineWidth', 2);
plot(x, mean(Acc_UDC,1), 'r--d', 'LineWidth', 2);
plot(x, mean(Acc_MUDC,1), 'r-d', 'LineWidth', 2);
plot(x, mean(Acc_HS,1), 'k-.s', 'LineWidth', 2);

xlim([5 40]);
ylim([0 30]);
xlabel('Dimensionality');
ylabel('Accuracy (%)');
legend({'OC', 'MFCC', 'DC', 'RDC', 'UDC', 'MUDC', 'HS'}, 'FontSize', 12);

%% Acc vs. polyphony of different features
% train on poly2lated signal
Best_Acc_OC = zeros(10, 5);
Best_Acc_MFCC = zeros(10, 5);
Best_Acc_DC = zeros(10, 5);
Best_Acc_RDC = zeros(10, 5);
Best_Acc_UDC = zeros(10, 5);
Best_Acc_MUDC = zeros(10, 5);
Best_Acc_HS = zeros(10, 5);

for poly = 2:6
    load(sprintf('results_trainiso/Acc_OC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_MFCC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_DC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_RDC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_UDC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_MUDC_poly%d.mat', poly));
    load(sprintf('results_trainiso/Acc_HS_poly%d.mat', poly));

    [~, idx] = max(mean(Acc_OC, 1));
    Best_Acc_OC(:, poly-1) = Acc_OC(:,idx);
    [~, idx] = max(mean(Acc_MFCC, 1));
    Best_Acc_MFCC(:, poly-1) = Acc_MFCC(:,idx);
    [~, idx] = max(mean(Acc_DC, 1));
    Best_Acc_DC(:, poly-1) = Acc_DC(:,idx);
    [~, idx] = max(mean(Acc_RDC, 1));
    Best_Acc_RDC(:, poly-1) = Acc_RDC(:,idx);
    [~, idx] = max(mean(Acc_UDC, 1));
    Best_Acc_UDC(:, poly-1) = Acc_UDC(:,idx);
    [~, idx] = max(mean(Acc_MUDC, 1));
    Best_Acc_MUDC(:, poly-1) = Acc_MUDC(:,idx);
    [~, idx] = max(mean(Acc_HS, 1));
    Best_Acc_HS(:, poly-1) = Acc_HS(:,idx);
end
figure; h = axes('FontSize', 16); hold on; grid on;
plot(2:6, mean(Best_Acc_OC,1), 'b--^', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_MFCC,1), 'b-^', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_DC,1), 'g--o', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_RDC,1), 'g-o', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_UDC,1), 'r--d', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_MUDC,1), 'r-d', 'LineWidth', 2);
plot(2:6, mean(Best_Acc_HS,1), 'k-.s', 'LineWidth', 2);
ylim([0 40]);
xlabel('Polyphony');
ylabel('Accuracy (%)');
legend({'OC', 'MFCC', 'DC', 'RDC', 'UDC', 'MUDC', 'HS'}, 'FontSize', 12);

%% calculate a confusion matrix of MUDC when poly=4
[trainLabel, trainFeature] = createTrainData('MUDC', 50, 5);
% scale the training data to [-1 1]
minTrainFeature = min(trainFeature, [], 1);
ranTrainFeature = max(trainFeature, [], 1) - min(trainFeature, [], 1);
scaledTrainFeature = 2*(trainFeature - repmat(minTrainFeature, size(trainFeature,1), 1))...
    ./ repmat(ranTrainFeature, size(trainFeature,1), 1) - 1;
% create test data
[testLabel, testFeature, testPitch] = createTestData_RWC('MUDC', 1000, 4, 50);
% scale the test data using the same linear transformation
scaledTestFeature = 2*(testFeature - repmat(minTrainFeature, size(testFeature,1), 1))...
    ./ repmat(ranTrainFeature, size(testFeature,1), 1) - 1;
% SVM train
model = svmtrain(trainLabel, scaledTrainFeature(:, 1:30), '-b 1 -c 100');
% SVM prediction
[predicted_label, accuracy, prob_estimates] = svmpredict(testLabel, scaledTestFeature(:, 1:30), model, '-b 1');
% save results
save('MUDC_poly4_results.mat', 'testLabel', 'predicted_label', 'accuracy');

confMat = zeros(13);
for i = 1:4000
    confMat(testLabel(i), predicted_label(i)) = confMat(testLabel(i), predicted_label(i)) + 1;
end

%% plot confusion matrix figure
figure;
imagesc(confMat);
colormap(flipud(gray));
set(gca, 'XTick', 1:13);
set(gca,'XTickLabel',{'Flute','Oboe','Bb.Cl.','Bn.','A.Sax','Tr.',...
    'Horn','T.Trb.','Tuba','Violin','Viola','Cello','Bass'},'FontSize',12);
set(gca, 'YTick', 1:13);
set(gca,'YTickLabel',{'Flute','Oboe','Bb.Cl.','Bn,','A.Sax','Tr.',...
    'Horn','T.Trb.','Tuba','Violin','Viola','Cello','Bass'},'FontSize',12);
% set(gca,'YTickLabel',{'Flute','Oboe','BbClar','Bassoon','AltoSax','Trumpet',...
%     'Horn','Trb.','Tuba','Violin','Viola.','Cello','Bass'});
% rotate tick labels
a=get(gca,'XTickLabel');
b=get(gca,'XTick');
text(b,repmat(13.7,length(b),1),a,'HorizontalAlignment','right','rotation',45,'fontsize',12);
% remove the old ticks
set(gca,'XTickLabel',{''});
xlabel('Predicted labels');
ylabel('Actual labels');


