function ceps = disceps(fk, ak, p, lambda)
% ceps = disceps(fk, ak, p, lambda)
% Calculate discrete cepstrum from spectral peaks
% Input:
%   - fk    : normalized frequencies of peaks (range from 0 to 1)
%   - ak    : amplitudes of peaks in dB
%   - p     : cepstrum order (there will be p ceps coefficients)
%   - labda : regularization factor
% Output:
%   - ceps  : the discrete cepstrum coefficients
%
% Author: Zhiyao Duan
% Created: 7/17/2012
% Last modified: 7/25/2013

if nargin < 4
    lambda = 1e-4;
end
if nargin < 3
    p = 30;
end

% convert row vectors to column vectors
if size(fk, 1) == 1
    fk = fk';
end
if size(ak, 1) == 1
    ak = ak';
end

% calculate cepstrum coefficients
L = length(fk);
M = zeros(L, p);

M(:,1) = 1;
M(:,2:end) = sqrt(2) * cos(2*pi * fk * (1:p-1));

R = 8 * pi^2 * (0:p-1).^2;
MtMR = M'*M + lambda/(1-lambda) * diag(R);

ceps = MtMR \ (M'*ak);