function [f0min, f0max] = estf0range(wavName)
% Get pitch range for non-piano notes of the University of IOWA dataset. It
% works with the specific naming convention that I'm using.
%
% Input
%   - wavName   : the (without path) filename of the segmented note
% Output
%   - f0min     : the minimum possible pitch in Hz
%   - f0max     : the maximum possible pitch in Hz
%
% Author: Zhiyao Duan
% Last modified: 11/4/2013

% define F0 range by reading wavName
idx = strfind(wavName, '_');
idx = [idx, strfind(wavName, '.')];
for i = 1:length(idx)
    temp = str2num(wavName(idx(i)+1 : idx(i+1)-1));
    if ~isempty(temp)
        noteIdx = temp;
        break;
    end
end
noteNameRange = wavName(idx(i-1)+1 : idx(i)-1);
for i = 1:length(noteNameRange)
    temp = str2num(noteNameRange(i));
    if ~isempty(temp)
        break;
    end
end
startNote = note2midinum(noteNameRange(1:i));
currNote = startNote + noteIdx - 1;

f0min = midi2hz(currNote-12);
f0max = midi2hz(currNote+12);

