function FisherRatio = fisherratio(Data, p)
% Calculate the Fisher score of data
%
% Input 
%   - Data          : a 1-d cell. Each element is a 2-d matrix. Each column of the
%                     matrix is a data instance.
%   - p             : only the first p dimensions are used 
% Output
%   - FisherRatio   : the fisher ratio, the concept is "interclass variance
%                       / total variance"
%
% Author: Zhiyao Duan
% Created: 8/19/2013
% Last modified: 11/4/2013

class = length(Data);       % number of classes
dim = size(Data{1},1);      % dimensionality

if nargin<2 p=dim; end

if dim<p 
    FisherRatio = 0; 
    return; 
end

% means of each class and the overall mean
miu_all = zeros(p, 1);
num_class = zeros(class, 1);
miu = zeros(p, 4);
for c = 1:class
    miu(:,c) = mean(Data{c}(1:p,:), 2);
    num_class(c) = size(Data{c}, 2);
    miu_all = miu_all + miu(:,c) * num_class(c);
end
miu_all = miu_all / sum(num_class);

Sb = zeros(p, p);       % interclass covariance matrix ('b' for 'between-class')
St = zeros(p, p);       % total covariance matrix ('t' for 'total-variance')
for c = 1:class
    Sb = Sb + num_class(c) * (miu(:,c)-miu_all) * (miu(:,c)-miu_all)';
    St = St + (Data{c}(1:p,:) - repmat(miu_all,1,num_class(c)))...
        * (Data{c}(1:p,:) - repmat(miu_all,1,num_class(c)))';
end
FisherRatio = trace(Sb / St); % formular: tr(Sb * inv(St))
