function F0HarmStruct = harmstruct(f0, peak, peakAmp, maxHarm)    
% Calculate the harmonic structure from spectral peaks and the given pitch
%
% Input
%   - f0            : the pitch in Hz
%   - peak          : peaks in Hz
%   - peakAmp       : peak amplitudes in dB
%   - maxHarm       : the maximum number of harmonics to consider in the
%                       harmonic structure (i.e. the dimensionality of the
%                       harmonic structure) 
% Output
%   - F0HarmStruct  : the calculated harmonic structure for the f0
%
% Author: Zhiyao Duan
% Last modified: 11/4/2013

Thresh_PeakF0Belong = 0.3;

mpeak = hz2midi(peak);                                	% peak frequency in MIDI number
fpeak = peak;                                           % peak frequency in Hz

mF0 = hz2midi(f0);                                      % F0 in MIDI number
fF0 = f0;                                               % F0 in Hz

Ratio = fpeak / fF0;                                    % calculate the harmonic number of peak i given F01
RRatio = round(Ratio);

% Use frequency deviation to decide which peak can belong to which F0
err = mpeak - mF0 - 12*log2(max(RRatio,1));             % the frequency deviation of all peaks from harmonics of F01

% Calculate the linear harmonic structure
F0HarmStruct = zeros(1, maxHarm);
for k = 1:maxHarm
    harmidx = find((abs(err)<Thresh_PeakF0Belong & (RRatio == k)) == 1);   % find all peaks that could belong to this harmonic
    if isempty(harmidx)
        F0HarmStruct(k) = 0;                            % if no peak is associated, use 0. This is much better!
    else
        [~, idx] = min(abs(err(harmidx)));
        F0HarmStruct(k) = 10^(peakAmp(harmidx(idx))/20);% if there are multiple peaks, use the closest one
    end
end

% normalize harmonic structure to have the total energy 100 dB for each F0's harmonics
idx = F0HarmStruct ~= 0;
tmpLinearHarmonic = F0HarmStruct(idx);
F0HarmStruct(idx) = 20*log10(tmpLinearHarmonic / sum(tmpLinearHarmonic) * 1e5);
    
