function ceps = unidisceps(fk, ak, p, fs, bMel)
% ceps = unidisceps(fk, ak, p, lambda)
% Calculate Uniform Discrete Cepstrum (UDC) from spectral peaks. This is a new
% (different) version than Rodet's algorithm. We removed the inverse matrix
% transformation. This algorithm is a strict analogy of cepstrum
% calculation.
%
% Input:
%   - fk    : normalized frequencies of peaks (range from 0 to 1)
%   - ak    : amplitudes of peaks in dB
%   - p     : cepstrum order (there will be p ceps coefficients)
%   - fs    : sampling rate, used to convert fk to mel-scale frequency
%   - bMel  : 1: convert fk to mel-scale frequency. (default: 0)
% Output:
%   - ceps  : the discrete cepstrum coefficients
%
% Author: Zhiyao Duan
% Created: 8/9/2012
% Last modified: 7/25/2013

if nargin<3  p = 30; end
if nargin<5  bMel = 0; end

if bMel == 1
    fk = 0.5*log10(1+fk*fs/700)/log10(1+fs/1400);
end

% convert row vectors to column vectors
if size(fk, 1) == 1
    fk = fk';
end
if size(ak, 1) == 1
    ak = ak';
end

% calculate cepstrum coefficients
L = length(fk);
M = zeros(L, p);

M(:,1) = 1;
M(:,2:end) = sqrt(2) * cos(2*pi * fk * (1:p-1));

ceps = M'*ak;